# $Id: install.py,v 1.4.6.30 2003/08/14 10:24:46 gitte Exp $
# $Source: /usr/local/cvs_root/MyMediaManager/Extensions/install.py,v $
__version__ = "$Revision: 1.4.6.30 $"[11:-2]

from Products.CMFCore.TypesTool import ContentFactoryMetadata
from Products.CMFCore.DirectoryView import addDirectoryViews, createDirectoryView
from Products.CMFCore.utils import getToolByName, minimalpath
#from Products.CMFBoringProduct import CMFBoringContent, boring_globals
from Products.MyMediaManager import mmm_globals, MediaClip, MasterFile, MediaEvent, Medianet, MediaCinema, MediaSkinExtensions
from Products.ExternalMethod import ExternalMethod
from App.Common import package_home
from cStringIO import StringIO
import string
import zLOG

out = StringIO()

type_classes = (MediaClip, MasterFile, MediaEvent, MediaCinema, MediaSkinExtensions)

def registerType(self, fti):
    """
        Register type defined in fti in the types tool
    """
    typestool = getToolByName(self, 'portal_types')
    if fti['id'] not in typestool.objectIds():
        cfm = apply(ContentFactoryMetadata, (), fti)
        typestool._setObject(fti['id'], cfm)
        out.write('Registered %s with the types tool\n' % fti['id'])
    else:
        out.write('Object "%s" already existed in the types tool\n' % (fti['id']))

def unregisterType(self, type):
    """
        Un-Register type in the types tool
    """
    typestool = getToolByName(self, 'portal_types')
    if type in typestool.objectIds():
        typestool._delObject(type)
        out.write('Un-Registered %s with the types tool\n' % type)
    else:
        out.write('Object "%s" did not exist in the types tool\n' % type)

def removeSkin(self, skinname):
    """
    Remove a skin from portal_skins
    """
    skinstool = getToolByName(self, 'portal_skins')
    skins = skinstool.getSkinSelections()
    for skin in skins:
        path = skinstool.getSkinPath(skin)
        path = map(string.strip, string.split(path,','))
        if skinname in path:
            path.remove(skinname)
            path = string.join(path, ', ')
            # addSkinSelection will replace existing skins as well.
            skinstool.addSkinSelection(skin, path)
            out.write("Removed %s from %s skin\n" % (skinname, skin))

    if skinname in skinstool.objectIds():
        skinstool._delObject(skinname)
        out.write('Removed %s from the skinstool\n' % skinname)

def install_plugins(self):
    """
    Install the skins for the plugins
    """
    portalroot = getToolByName(self, 'portal_url').getPortalObject()
    typestool = getToolByName(self, 'portal_types')
    skinstool = getToolByName(self, 'portal_skins')
    portal_mediamanager = getToolByName(self, 'portal_mediamanager')

    pluginlist = portal_mediamanager.listRegisteredPlugins()
    typeinfo_list = typestool.listDefaultTypeInformation()
    for plugindict in pluginlist:
        plugintype = plugindict['type']
        skinname = plugindict['skinname']
        skinpath = plugindict['skinpath']
        for typeinfo_name, type_content in typeinfo_list:
            zLOG.LOG('install', zLOG.INFO, 'Typeinfo name %s - content: %s' % (typeinfo_name, str(type_content)))
            is_type = string.find(type_content['id'], plugintype)
            if is_type != -1:
                exists = 0
                for cti in typestool.listTypeInfo():
                    if cti.Title() == plugintype:
                        exists = 1
                if not exists:
                    typestool.manage_addTypeInformation(add_meta_type='Factory-based Type Information', id=None, typeinfo_name=typeinfo_name)
                    out.write('Registered %s in the types tool\n' % typeinfo_name)
                else:
                    out.write('%s already registered in the types tool\n' % typeinfo_name)
        # Setup the skins
        if skinname not in skinstool.objectIds():
            # We need to add Filesystem Directory Views for any directories
            # in our skins/ directory.  These directories should already be
            # configured.
            fp = minimalpath(skinpath)
            createDirectoryView(skinstool, fp, skinname)
            out.write("Added '%s' directory view to portal_skins\n" % skinname)
        else:
            out.write("'%s' directory view already exists in portal_skins\n" % skinname)

        # Now we need to go through the skin configurations and insert
        # 'CMFBoringContent' into the configurations.  Preferably, this
        # should be right before where 'content' is placed.  Otherwise, we
        # append it to the end.
        skins = skinstool.getSkinSelections()
        for skin in skins:
            path = skinstool.getSkinPath(skin)
            path = map(string.strip, string.split(path,','))
            if skinname not in path:
                try:
                    path.insert(path.index('mmm'), skinname)
                except ValueError:
                    path.append(skinname)

                path = string.join(path, ', ')
                # addSkinSelection will replace existing skins as well.
                skinstool.addSkinSelection(skin, path)
                out.write("Added '%s' to %s skin\n" % (skinname, skin))
            else:
                out.write("Skipping %s skin, '%s' is already set up\n" % (skin, skinname))

def install(self):
    """Register all MMMCore components with the necessary tools"""
    out.write('MyMediaManager "New Generation" Installation\n')
    out.write('===========================\n\n')
    skinstool = getToolByName(self, 'portal_skins')
    typestool = getToolByName(self, 'portal_types')
    actionstool = getToolByName(self, 'portal_actions')
    portalroot = getToolByName(self, 'portal_url').getPortalObject()
    portalcatalog = getToolByName(self, 'portal_catalog')

    for c in type_classes:
        t = c.factory_type_information
        if type(t) == type(()):
            for fti in t:
                registerType(self, fti)
        else:
            registerType(self, t)

    tools = {'portal_medianet':'Media Manager Medianet',}

    for tool in tools.keys():
        if not tool in portalroot.objectIds():
            portalroot.manage_addProduct['MyMediaManager'].manage_addTool(tools[tool], None)
            out.write('Created %s\n' % tools[tool])
        else:
            out.write('%s already exists\n' % tools[tool])

    ap = actionstool.listActionProviders()
    for tool in tools.keys():
        if (type(ap) is not type([]) and type(ap) is not type(())) or not tool in ap:
            toolobj = getToolByName(self, tool)
            if hasattr(toolobj, '_actions'):
                try:
                    actionstool.addActionProvider(tool)
                    out.write('Added %s as action provider\n' % tools[tool])
                except:
                    out.write('Failed adding %s as action provider. Please do it manually\n' % tools[tool])
            else:
                out.write('%s provides no actions\n' % tools[tool])
        else:
            out.write('%s already existed as action provider\n' % tools[tool])

    if not 'getMemberPass' in portalroot.objectIds():
        i = ExternalMethod.ExternalMethod('getMemberPass', 'Returns a members password', 'MyMediaManager.utils', 'getUserPass')
        portalroot._setObject('getMemberPass', i)
        out.write('Created getMemberPass method\n')
    else:
        out.write('getMemberPass method already existed\n')

    if not 'transitions' in portalroot.objectIds():
        i = ExternalMethod.ExternalMethod('transitions', 'Get transitions from portal_workflow', 'MyMediaManager.utils', 'getTransitions')
        portalroot._setObject('transitions', i)
        out.write('Created transitions method\n')
    else:
        out.write('transitions method already existed\n')

    if not 'start' in portalcatalog.Indexes.objectIds():
        portalcatalog.addIndex('start', 'FieldIndex')
        out.write('Added start as FieldIndex\n')
    else:
        out.write('start already exists as FieldIndex\n')

    if not 'end' in portalcatalog.Indexes.objectIds():
        portalcatalog.addIndex('end', 'FieldIndex')
        out.write('Added end as FieldIndex\n')
    else:
        out.write('end already exists as FieldIndex\n')

    # Setup the skins
    if 'mmm' not in skinstool.objectIds():
        # We need to add Filesystem Directory Views for any directories
        # in our skins/ directory.  These directories should already be
        # configured.
        fp = minimalpath("%s/zpt_skins" % package_home(mmm_globals))
        createDirectoryView(skinstool, fp, 'mmm')
        #addDirectoryViews(skinstool, 'zpt_skins', fp)
        out.write("Added 'mmm' directory view to portal_skins\n")
    else:
        out.write("'mmm' directory view already exists in portal_skins\n")

    # Now we need to go through the skin configurations and insert
    # 'CMFBoringContent' into the configurations.  Preferably, this
    # should be right before where 'content' is placed.  Otherwise, we
    # append it to the end.
    skins = skinstool.getSkinSelections()
    for skin in skins:
        path = skinstool.getSkinPath(skin)
        path = map(string.strip, string.split(path,','))
        if 'mmm' not in path:
            try: path.insert(path.index('content'), 'mmm')
            except ValueError:
                path.append('mmm')

            path = string.join(path, ', ')
            # addSkinSelection will replace existing skins as well.
            skinstool.addSkinSelection(skin, path)
            out.write("Added 'mmm' to %s skin\n" % skin)
        else:
            out.write("Skipping %s skin, 'mmm' is already set up\n" % (skin))
            
    #install_plugins(self)

    return out.getvalue()

def uninstall(self):
    """
    Remove skins, types and special objects
    """
    out.write('MyMediaManager Un-Installation\n')
    out.write('===========================\n\n')
    portalroot = getToolByName(self, 'portal_url').getPortalObject()
    for c in type_classes:
        t = c.factory_type_information
        if type(t) == type(()):
            for fti in t:
                unregisterType(self, fti['id'])
        else:
            unregisterType(self, t['id'])

    removeSkin(self, 'mmm')

    if 'portal_mediamanager' in portalroot.objectIds():
        portalroot._delObject('portal_mediamanager')
        out.write('Removed Media Manager Tool\n')

    if 'medianet' in portalroot.objectIds():
        portalroot._delObject('medianet')
        out.write('Removed Medianet\n')

    return out.getvalue()
