##############################################################################
#    Copyright (C) 2001-2002  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################
"""
"""

ADD_CONTENT_PERMISSION = 'Add portal content'

#Globals
import zLOG, string, os, sys
import Globals
from DateTime.DateTime import DateTime
from Acquisition import aq_parent, aq_inner, aq_base
from AccessControl import ClassSecurityInfo
from OFS.content_types import guess_content_type

#CMF
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.utils import getToolByName
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFDefault.utils import formatRFC822Headers

from ServerInfo import ServerInfo
from utils import ftp_upload, StatusFile

factory_type_information = ({'id': 'Master File',
                                                  'meta_type': 'Media Manager Master File',
                                                  'description': 'Use master file to store link to files',
                                                  'product': 'MyMediaManager',
                                                  'icon': 'masterfile_icon.png',
                                                  'factory': 'addMasterFile',
                                                  'filter_content_types': 0,
                                                  'immediate_view': 'masterfile_edit_form',
                                                  'actions':({'name': 'View',
                                                                   'id': 'view',
                                                                   'action': 'masterfile_view',
                                                                   'permissions': ('View',),
                                                                   'category': 'object'},
                                                                  {'name': 'Edit',
                                                                   'edit': 'edit',
                                                                   'action': 'masterfile_edit_form',
                                                                   'permissions': ('Modify portal content',)})
                                                  },)

class MasterFile( PortalContent, DefaultDublinCoreImpl ):
    """
        A Masterfile is logical description of a media file
    """

    meta_type='Media Manager Master File'
    portal_type='Object'
    effective_date = expiration_date = None
    _isDiscussable = 1
    security = ClassSecurityInfo()

    def __init__(self, id):
        DefaultDublinCoreImpl.__init__(self)
        self.id = id
        self.title = ''
        self.description = ''
        self._filename = ''

    security.declarePublic('DumpLOG')
    def DumpLOG(self, REQUEST):
        zLOG.LOG("MasterFile", 0, "%s" % (str(REQUEST)))

    def SearchableText(self):
        """
            text for indexing
        """
        return "%s %s" % (self.title, self.description)
    
    def manage_beforeDelete(self, item, container):
        """
        Manage before delete - delete media files on server if set
        """
        medianet = getToolByName(self, 'portal_medianet')
        if medianet.cleanupFiles():
            zLOG.LOG("%s - beforeDelete" % self.meta_type, zLOG.INFO, 'Cleaning up files')
            serverinfos = self.listServerInfos(stype='on-demand')
            import ftplib
            _ftp = ftplib.FTP()
            _ftp.debugging = 1
            for serverinfo in serverinfos:
                mserver = serverinfo.getMediaServer()
                zLOG.LOG("%s - beforeDelete" % self.meta_type, zLOG.INFO, 'Cleaning up for %s' % mserver.title_or_id())
                host = mserver.getHost()
                port = 21
                login = mserver.getUploadLogin()
                password = mserver.getUploadPassword()
                remote_directory = mserver.getUploadDir()
                remote_name = '%s%s' % (self.getFileName(), mserver.getExtension())
                zLOG.LOG("%s - beforeDelete" % self.meta_type, zLOG.INFO, 'FTP info is:\n' + \
                    'Host: %s\nPort: %d\nLogin: %s\nPassword: %s\nDir: %s\nFile: %s' % \
                    (host, port, login, password, remote_directory, remote_name))
                try:
                    _ftp.connect(host, port)
                    zLOG.LOG("%s - beforeDelete" % self.meta_type, zLOG.INFO, 'Connected')
                except ftplib.all_errors:
                    zLOG.LOG("%s - beforeDelete" % self.meta_type, zLOG.ERROR, \
                        'Not connected', error=sys.exc_value)
                    _ftp.quit()
                    continue
                try:
                    _ftp.login(login, password)
                    zLOG.LOG("%s - beforeDelete" % self.meta_type, zLOG.INFO, 'Logged in')
                except ftplib.all_errors:
                    zLOG.LOG("%s - beforeDelete" % self.meta_type, zLOG.ERROR, \
                        'Not Logged in', error=sys.exc_value)
                    _ftp.quit()
                    continue

                # normalize remote_name and remote_path
                slashidx = string.rfind(remote_name, '/')
                if slashidx != -1:
                    p = remote_name[:slashidx]
                    remote_directory = string.join((remote_directory, p), '/')
                    remote_name = remote_name[slashidx+1:]
                    zLOG.LOG("%s - beforeDelete" % self.meta_type, \
                       zLOG.INFO, 'Fixing name and path: [%s] [%s]' % (remote_directory, remote_name))
            
                if remote_directory:
                    currentpath = _ftp.pwd()
                    try:
                        _ftp.cwd(remote_directory)
                        _ftp.cwd(currentpath)
                        zLOG.LOG("%s - beforeDelete" % self.meta_type, \
                            zLOG.INFO, 'ftp_dir_exists: yes!')
                    except:
                        zLOG.LOG("%s - beforeDelete" % self.meta_type, \
                            zLOG.INFO, 'ftp_dir_exists: exception', error=sys.exc_info()[1])
                        # we assume it doesn't exist
                        _ftp.quit()
                        continue
                    try:
                        _ftp.cwd(remote_directory)
                        zLOG.LOG("%s - beforeDelete" % self.meta_type, \
                            zLOG.INFO, 'CWD to %s' % remote_directory)
                    except:
                        zLOG.LOG("%s - beforeDelete" % self.meta_type, \
                            zLOG.ERROR, 'Could not CWD to %s' % remote_directory, \
                            error = sys.exc_value)
                        _ftp.quit()
                        continue
                    
                try:
                    _ftp.size(remote_name)
                    try:
                        _ftp.delete(remote_name)
                        zLOG.LOG("%s - beforeDelete" % self.meta_type, \
                            zLOG.INFO, 'ftpdelete: %s deleted' % remote_name)
                    except ftplib.error_perm:
                        zLOG.LOG("%s - beforeDelete" % self.meta_type, \
                            zLOG.ERROR, 'ftpdelete: exception permission', error=sys.exc_info())
                        continue
                    except ftplib.error_reply:
                        zLOG.LOG("%s - beforeDelete" % self.meta_type, \
                            zLOG.ERROR, 'ftpdelete: exception reply', error=sys.exc_info())
                        continue
                    except:
                        zLOG.LOG("%s - beforeDelete" % self.meta_type, \
                            zLOG.ERROR, 'ftpdelete: unknown exception', error=sys.exc_info())
                        continue
                
                except:
                    # there is no such file
                    zLOG.LOG("%s - beforeDelete" % self.meta_type, \
                        zLOG.ERROR, 'ftpdelete: no such file', error=sys.exc_info())
                    continue
                zLOG.LOG("%s - beforeDelete" % self.meta_type, zLOG.INFO, \
                    'Finished deleting file for server %s' % mserver.title_or_id())
                _ftp.quit()
            _ftp = None
            zLOG.LOG("%s - beforeDelete" % self.meta_type, zLOG.INFO, 'Logged out')
        PortalContent.manage_beforeDelete(self, item, container)

    def edit(self, title=None, description=None, filename=None, REQUEST=None):
        """
        Edit the metadata for the masterfile
        """
        if title is not None:
            self.setTitle(title)

        if description is not None:
            self.setDescription(description)

        if filename is not None:
            self._filename = filename

        self.reindexObject()
        msg = 'Masterfile updated'
        if REQUEST:
            req = REQUEST
            req_url = req.get('HTTP_REFERER', None)
            if not req_url:
                req_url = "%s/masterfile_edit_form" % self.absolute_url()
            req.RESPONSE.redirect('%s?portal_status_message=%s' % (req_url, string.replace(msg, ' ', '+')))

    security.declareProtected(CMFCorePermissions.ManageProperties, 'upload')
    def upload(self, data, serverinfo):
        """
        Upload data to a mediaserver
        """
        resp = ['', '']
        server = serverinfo.getMediaServer()
        if server.getUploadLogin() != '' and server.getUploadPassword() != '':
            filename = "%s%s" % (self.getFileName(), server.getExtension())
            resp = ftp_upload(file=data, host=server.getHost(), login=server.getUploadLogin(), password=server.getUploadPassword(), remote_directory=server.getUploadDir(), remote_name=filename)
            #zLOG.LOG('Masterfile - upload', zLOG.INFO, 'Upload resp: %s' % str(resp))
        else:
            resp[1] = "FTP username or password empty"

        content_type = self._get_content_type(data, data.read(100), self.id)
        #zLOG.LOG('Masterfile', zLOG.INFO, 'Content type: %s' % content_type)
        edit_stuff = {'content_type': content_type, 'isEnabled': 1}
        serverinfo.edit(datadict=edit_stuff)
        return resp[1]
    
    security.declareProtected(CMFCorePermissions.ManageProperties, 'retrieveFile')
    def retrieveFile(self, serverinfo, encoderserver):
        """
        Retrieve a mediafile from a remote encoder server
        """
        memtool = getToolByName(self, 'portal_membership')
        username = memtool.getAuthenticatedMember().getUserName()
        password = memtool.getAuthenticatedMember().getPassword()
        endpath = "%s?serverinfoid=%s" % (self.absolute_url(), serverinfo.getId())
        server = serverinfo.getMediaServer()
        filename = "%s%s" % (self.getFileName(), server.getExtension())
        encoderserver.doUpload(endpath, username, password, filename, serverinfo)

    def PUT(self, REQUEST, RESPONSE):
        """
        Handle HTTP PUT requests
        """
        server = None
        zLOG.LOG(self.meta_type, zLOG.INFO, 'PUT called')

        # in this version we propably have to do it another way (thomas)
        if REQUEST.has_key('QUERY_STRING') and string.strip(REQUEST.get('QUERY_STRING', None)) !='':
            #zLOG.LOG('Masterfile put', zLOG.INFO, 'QUERY_STRING:\n%s' % str(REQUEST['QUERY_STRING']))
            query = REQUEST.get('QUERY_STRING')
            query = string.replace(query, "'", '')
            path, value = string.split(query, '=')
            if value.find('/') != -1:
                server = self.restrictedTraverse(value)
                serverinfo = self.getServerInfo(serverid=server.getId())
            else:
                serverinfo = self.getServerInfo(infoid=value)
                server = serverinfo.getMediaServer()
            zLOG.LOG('Masterfile put', zLOG.INFO, 'Path is: %s. Value is: %s' % (path, value))
            zLOG.LOG('Masterfile put', zLOG.INFO, 'Server is %s' % str(server))
        else:
            zLOG.LOG('Masterfile put', zLOG.INFO, 'REQUEST:\n%s' % str(REQUEST))

        self.dav__init(REQUEST, RESPONSE)
        self.state = 'uploading'
        content_type = REQUEST.get_header('content-type', None)
        content_length = REQUEST.get_header('Content-Length', None)
        #fn = string.split(REQUEST['HTTP_CONTENT_DISPOSITION'],'=') # if we ever need the original filename (thomas)
        
        instream = StatusFile(REQUEST['BODYFILE'], serverinfo, "ftp_written")
        if not self.getFileName():
            self.state = 'error'
            RESPONSE.setStatus(401, 'No Filename')
            return RESPONSE

        if server: # we have no garantee that there is a logic by now
            try:
                filename = '%s%s' % (self.getFileName(), server.getExtension())
                resp = ftp_upload(file=instream, host=server.getHost(), login=server.getUploadLogin(), password=server.getUploadPassword(), remote_directory=server.getUploadDir(), remote_name=filename)
            except:
                zLOG.LOG('Masterfile.PUT', zLOG.INFO, 'An error occured before ftp upload')
                resp = [250]
                zLOG.LOG(self.meta_type, zLOG.ERROR, \
                    'Exception in getting stat', error=sys.exc_info())
                #zLOG.LOG('Masterfile.PUT', zLOG.INFO, 'Sys: %s' % str(sys.exc_info()[1]))
        if resp[0] < 207 or resp[0] >= 300:
            #zLOG.LOG('Masterfile.PUT', zLOG.INFO, 'WE RULE TO BAD!!!!!')
            self.state = 'archived'
            RESPONSE.setStatus(resp[0])
            edit_stuff = {'content_type': content_type}
            serverinfo = self.getServerInfo(serverid=server.getId())
            serverinfo.edit(datadict=edit_stuff)
            return RESPONSE
        else:
            #zLOG.LOG('Masterfile.PUT', zLOG.INFO, 'We have no logic whatsoever')
            self.status = 'error'
            RESPONSE.setStatus(401)
            return RESPONSE
        edit_stuff = {'content_type': content_type, 'isEnabled': 1}
        serverinfo = self.getServerInfo(serverid=server.getId())
        serverinfo.edit(datadict=edit_stuff)
        RESPONSE.setStatus(204)
        return RESPONSE

    security.declarePrivate('_get_content_type')
    def _get_content_type(self, file, body, id, content_type=None):
        """
        Determine the mime-type
        """
        headers=getattr(file, 'headers', None)
        if headers and headers.has_key('content_type'):
            content_type=headers['content_type']
        else:
            if type(body) is not type(''): body=body.data
            content_type, enc=guess_content_type(getattr(file, 'filename', id), body, content_type)
        return content_type

    security.declarePublic('getFileName')
    def getFileName(self):
        """
        Return the filename (without extension) for the Masterfile
        """
        return self._filename

    security.declareProtected(CMFCorePermissions.View, 'getDefaultOndemandServerId')
    def getDefaultOndemandServerId(self):
        """
        Return the def. server ondemand id
        """
        server = self.getDefaultOndemandServer()
        if server:
            return server.getId()
        else:
            return None

    security.declareProtected(CMFCorePermissions.View, 'getDefaultOndemandServer')
    def getDefaultOndemandServer(self):
        """
        Return the def. server ondemand id
        """
        serverinfos = self.listServerInfos()
        medianet = getToolByName(self, 'portal_medianet')
        archiveservers = medianet.listServers('on-demand')
        ondemand_server = None
        for sinfo in serverinfos:
            mserver = sinfo.getMediaServer()
            if mserver in archiveservers:
                ondemand_server = mserver
                break
        
        if not ondemand_server:
           ondemand_server = medianet.getDefaultServer('ondemand')
        return ondemand_server

    security.declareProtected(CMFCorePermissions.View, 'getDefaultLiveServerId')
    def getDefaultLiveServerId(self):
        """
        Return the def. server ondemand id
        """
        server = self.getDefaultLiveServer()
        if server:
            return server.getId()
        else:
            return None

    security.declareProtected(CMFCorePermissions.View, 'getDefaultOndemandServer')
    def getDefaultLiveServer(self):
        """
        Return the def. server ondemand id
        """
        serverinfos = self.listServerInfos()
        medianet = getToolByName(self, 'portal_medianet')
        liveservers = medianet.listServers('live')
        liveserver = None
        for sinfo in serverinfos:
            mserver = sinfo.getMediaServer()
            if mserver in liveservers:
                liveserver = mserver
                break
        if not liveserver:
           liveserver = medianet.getDefaultServer('live')
        return liveserver

    security.declarePublic('getServerInfo')
    def getServerInfo(self, infoid=None, serverid=None):
        """
        Return the requestedServerInfo object
        """
        #zLOG.LOG('Master file', zLOG.INFO, 'Infoid is %s' % str(infoid))
        #zLOG.LOG('Master file', zLOG.INFO, 'serverid is %s' % str(serverid))
        medianet = getToolByName(self, 'portal_medianet')
        event = medianet.getEventFor(self)
        if not infoid and serverid:
            #zLOG.LOG('Master file', zLOG.INFO, 'Using serverid')
            for serverinfo in self.listServerInfos():
                server = serverinfo.getMediaServer()
                if server:
                    mediaid = server.getId()
                    if mediaid == serverid:
                        infoid = serverinfo.id
        serverinfo = getattr(event, infoid, None)
        #zLOG.LOG('Master file', zLOG.INFO, 'serverinfo is %s' % str(serverinfo))
        if not serverinfo:
            if serverid:
                mediaserver = medianet.getServerById(serverid)
            else: # XXX this method doesn't exist!!!
                mediaserver = medianet.getDefaultServerFor(self)
            #zLOG.LOG('Master file', zLOG.INFO, 'mediaserver is %s' % str(mediaserver))
            infoid = self.assignServer(mediaserver)
            #zLOG.LOG('Master file', zLOG.INFO, 'infoid is %s' % str(infoid))
            serverinfo = getattr(event, infoid)
            #zLOG.LOG('Master file', zLOG.INFO, 'serverinfo is %s' % str(serverinfo))
        return serverinfo

    security.declarePublic('listServerInfos')
    def listServerInfos(self, stype=None):
        """
        Return a list of ServerInfo Objects this masterfile uses.
        stype can be either 'on-demand' or 'live' or None meaning all.
        """
        medianet = getToolByName(self, 'portal_medianet')
        event = medianet.getEventFor(self)
        serverinfos = event.objectValues('Media Manager ServerInfo')
        if stype is not None:
            servers = medianet.listServers(stype)
            infos = []
            for info in serverinfos:
                if not info.getMediaServer():
                    try:
                        event.manage_delObjects(info.getId())
                    except:
                        pass
                elif info.getMediaServer() in servers:
                    infos.append(info)
            return infos
        else:
            return serverinfos

    security.declarePublic('listAssignedServers')
    def listAssignedServers(self, stype=None):
        """
        Return a list of Server Objects this masterfile uses meaning where a server info object
        has been instantiated.
        stype can be either 'on-demand' or 'live' or None meaning all.
        """
        infos = self.listServerInfos(stype)
        servers = []
        for info in infos:
            servers.append(info.getMediaServer())
        return servers

    security.declarePublic('listUnAssignedServers')
    def listUnAssignedServers(self, stype=None):
        """
        Return a list of Server Objects this masterfile doesn't use meaning where a server info object
        has not been instantiated.
        stype can be either 'on-demand' or 'live' or None meaning all.
        """
        medianet = getToolByName(self, 'portal_medianet')
        allservers = medianet.listServers(stype)
        assignedservers = self.listAssignedServers(stype)
        servers = []
        for server in allservers:
            if not server in assignedservers:
                servers.append(server)
        return servers

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'assignServer')
    def assignServer(self, mediaserver):
        """
        Assign a mediaserver to a masterfile.
        This is done by creating a new ServerInfo object
        """
        medianet = getToolByName(self, 'portal_medianet')
        event = medianet.getEventFor(self)
        if type(mediaserver) == type(''):
            mediaserver = medianet.getServerById(mediaserver)
        infoob = ServerInfo(self, mediaserver.getId())
        id = infoob.id
        event._setObject(id, infoob)
        serverinfo = getattr(event, id)
        datalist = mediaserver.getDataListForServerInfo()
        #zLOG.LOG('Maserfile', zLOG.INFO, 'Datalist is %s' % str(datalist))
        datadict = {}
        for dict in datalist:
            id = dict['id']
            value = dict['value']
            datadict[id] = dict['value']
            #zLOG.LOG('Master File - assign server', zLOG.INFO, 'Data line: %s = %s' % (str(id), str(value)))
        serverinfo.edit(datadict=datadict)
        return serverinfo.id

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'removeServerInfo')
    def removeServerInfo(self, infoid=None, serverid=None, REQUEST=None):
        """
        Removes an assigned a mediaserver from masterfile.
        This is done by removing ServerInfo object
        """
        medianet = getToolByName(self, 'portal_medianet')
        event = medianet.getEventFor(self)
        if infoid is not None:
            delid = infoid
        elif serverid is not None:
            for serverinfo in event.objectValues('Media Manager ServerInfo'):
                mserver = serverinfo.getMediaServer()
                if mserver.getId() == serverid:
                    delid = serverinfo.id
                    break
        else:
            raise AttributeError, 'You did not specify either server id or info id'

        if delid in event.objectIds():
            event._delObject(delid)
        else:
            zLOG.LOG('Masterfile - remove server', zLOG.INFO, 'Del id NOT found in event')

        if REQUEST is not None and REQUEST.has_key('return_path'):
            url = REQUEST.get('return_path')
            msg = "Deleted"
            REQUEST.RESPONSE.redirect('%s?portal_status_message=%s' % (url, msg.replace(' ', '+')))

    def cb_isMoveable(self):
        return 0

    def cb_isCopyable(self):
        """
        You can copy this object
        """
        return 0

def addMasterFile(self, id, REQUEST=None):
    ob=MasterFile(id)
    self._setObject(id, ob)

Globals.InitializeClass(MasterFile)
