##############################################################################
#    Copyright (C) 2001  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################
"""
"""

ADD_CONTENT_PERMISSION = 'Add portal content'

#Globals
import Globals
import zLOG
from AccessControl import ClassSecurityInfo
from DateTime.DateTime import DateTime
from Acquisition import aq_parent, aq_inner, aq_base

#CMF
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.CMFCorePermissions import View
from Products.CMFCore.CMFCorePermissions import ManageProperties
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.utils import getToolByName
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl

#MMMCore
from General import TimeStamp
import General

factory_type_information = ({'id': 'Media Clip',
      'meta_type': 'Media Manager Media Clip',
      'description': 'Use clip to index your masterfiles',
      'product': 'MyMediaManager',
      'icon': 'clip_icon.png',
      'factory': 'addMediaClip',
      'filter_content_types': 0,
      'immediate_view': 'mediaclip_edit_form',
      'actions':
          ({'name': 'View',
            'id': 'view',
            'action': 'mediaclip_view',
        'permissions': ('View',),
        'category': 'object'},
           {'name': 'Edit',
            'id': 'edit',
            'action': 'mediaclip_edit_form',
        'permissions': ('Modify portal content',),
        'category': 'object'})
       },)


class MediaClip(PortalContent, DefaultDublinCoreImpl):
    """
        A media clip
    """

    meta_type='Media Manager Media Clip'
    portal_type='Object'
    effective_date = expiration_date = None
    _isDiscussable = 1
    security = ClassSecurityInfo()

    def __init__(self, id):
        DefaultDublinCoreImpl.__init__(self)
        self.id = id
        self._tcin = TimeStamp()
        self._tcout = TimeStamp()
        self._masterfile_path = None

    #security.declareProtected(CMFCorePermissions.View, 'getTcIn')
    security.declarePublic('getTcIn')
    def getTcIn(self, context='cinema', serverinfo=None):
        """
        Return the tcin for the media clip
        """
        req = self.REQUEST
        serverinfo = None
        mnet = getToolByName(self, 'portal_medianet')
        #zLOG.LOG("%s - get Tc in" % self.meta_type, zLOG.INFO, 'Context is %s' % context)
        if not serverinfo:
            if context == 'cinema':
                serverinfo = mnet.getServerinfoFor(self)
            else:
                serverinfo = mnet.getEditServerInfo(self)
        zLOG.LOG("%s - get Tc in" % self.meta_type, zLOG.INFO, 'Server used is %s - %s' % (serverinfo.getMediaServer().title, serverinfo.getMediaServer().meta_type))

        """
        if req.cookies.has_key('pref_edit_format'):
            serverinfo = mnet.getEditServerInfo(self)
        else:
            serverinfo = mnet.getServerinfoFor(self)
        """

        event = mnet.getEventFor(self)
        eventstart = event.start()
        eventstart = eventstart.strftime('%Y/%m/%d %H:%M:%S')
        zLOG.LOG("%s - get Tc in" % self.meta_type, zLOG.INFO, 'Eventstart is %s' % str(eventstart))
        eventstart = DateTime(eventstart).millis()
        zLOG.LOG("%s - get Tc in" % self.meta_type, zLOG.INFO, 'Eventstart millis is %s' % str(eventstart))
        encstart = serverinfo.getEncoderStart()
        if not encstart:
            encstart = DateTime(event.start().strftime('%Y/%m/%d %H:%M:%S'))
        else:
            encstart = DateTime(encstart.strftime('%Y/%m/%d %H:%M:%S'))
        encstart = encstart.millis()
        offset = eventstart - encstart
        zLOG.LOG("%s - get Tc in" % self.meta_type, zLOG.INFO, 'Offset is %s' % str(offset))
        tcin = getattr(self, '_tcin')
        tcin = tcin.millis()
        tcin = offset + tcin
        tcin = TimeStamp(tcin)
        zLOG.LOG("%s - get Tc in" % self.meta_type, zLOG.INFO, 'Tc in is %s' % str(tcin))
        return tcin

    #security.declareProtected(CMFCorePermissions.View, 'getTcOut')
    security.declarePublic('getTcOut')
    def getTcOut(self, context='cinema', serverinfo=None):
        """
        Return the tcout for the media clip
        """
        req = self.REQUEST
        serverinfo = None
        mnet = getToolByName(self, 'portal_medianet')
        if not serverinfo:
            if context == 'cinema':
                serverinfo = mnet.getServerinfoFor(self)
            else:
                serverinfo = mnet.getEditServerInfo(self)

        """
        if req.cookies.has_key('pref_edit_format'):
            serverinfo = mnet.getEditServerInfo(self)
        else:
            serverinfo = mnet.getServerinfoFor(self)
        """

        event = mnet.getEventFor(self)
        eventstart = event.start()
        eventstart = eventstart.strftime('%Y/%m/%d %H:%M:%S')
        eventstart = DateTime(eventstart).millis()
        encstart = serverinfo.getEncoderStart()
        if not encstart:
            encstart = DateTime(event.start().strftime('%Y/%m/%d %H:%M:%S'))
        else:
            encstart = DateTime(encstart.strftime('%Y/%m/%d %H:%M:%S'))
        encstart = encstart.millis()
        offset = eventstart - encstart
        tcout = getattr(self, '_tcout')
        tcout = tcout.millis()
        tcout = offset + tcout
        return TimeStamp(tcout)

    security.declareProtected(CMFCorePermissions.View, 'setMasterFilePath')
    def setMasterFilePath(self, masterfile_path=None):
        """
        Sets the master file to the one specified in masterfile.
        Param:masterfilee - a string with the ful path to the masterfile object
        """
        if masterfile_path is not None:
            self._masterfile_path = masterfile_path

    security.declarePublic('getMasterFilePath')
    def getMasterFilePath(self):
        """
        Return the path to the master file the clip belongs to.
        """
        purl = getToolByName(self, 'portal_url').getPortalPath()
        mpath = getattr(self, '_masterfile_path', None)
        if mpath:
            if mpath[0] != '/':
                mpath = "%s/%s" % (purl, mpath)
            else:
                mpath = "%s%s" % (purl, mpath)
        return mpath

    security.declarePublic('getMasterFile')
    def getMasterFile(self):
        """
        Return the masterfile as an object
        """
        mpath = self.getMasterFilePath()
        if mpath:
            return self.restrictedTraverse(mpath)
        else:
            return None

    security.declarePublic('getStartTime')
    def getStartTime(self):
        """
        Return the start time relative to offsets
        """
        return self.getTcIn()

    security.declarePublic('getDuration')
    def getDuration(self):
        """
        Return duration
        """
        tcin = self.getTcIn()
        tcout = self.getTcOut()
        dur = tcout - tcin
        #zLOG.LOG('Clip', zLOG.INFO, 'Duration is: %s' % str(dur))
        return dur

    def setTimeCodes(self, tcin=None, tcout=None, relative=0):
        """
        Sets the Time clip data.
        """
        if relative:
            offset = 0
        else:
            mnet = getToolByName(self, 'portal_medianet')
            serverinfo = mnet.getEditServerInfo(self)
            #zLOG.LOG("%s - set time codes" % self.meta_type, zLOG.INFO, 'Using server: %s' % serverinfo.getMediaServer().title)
            event = mnet.getEventFor(self)
            eventstart = event.start()
            eventstart = eventstart.strftime('%Y/%m/%d %H:%M:%S')
            eventstart = DateTime(eventstart).millis()
            zLOG.LOG("%s - set time codes" % self.meta_type, zLOG.INFO, 'Event start millis is %s' % str(eventstart))
            encstart = serverinfo.getEncoderStart()            
            if not encstart:
                encstart = DateTime(event.start().strftime('%Y/%m/%d %H:%M:%S'))
            else:
                encstart = DateTime(encstart.strftime('%Y/%m/%d %H:%M:%S'))
            encstart = encstart.millis()
            zLOG.LOG("%s - set time codes" % self.meta_type, zLOG.INFO, 'Enc start millis is %s' % str(encstart))
            offset = eventstart - encstart
            zLOG.LOG("%s - set time codes" % self.meta_type, zLOG.INFO, 'Offset is %s' % str(offset))
        if tcin is not None:
            #zLOG.LOG("%s - set time codes" % self.meta_type, zLOG.INFO, 'Got tcin. Tcin is: %s' % str(tcin))
            if General.isinstance(tcin, TimeStamp):
                #zLOG.LOG('Media Clip', zLOG.INFO, 'Tcin is timestamp')
                tcin = tcin.millis()
                tcin = tcin - offset
                tcin = TimeStamp(tcin)
                zLOG.LOG("%s - set time codes" % self.meta_type, zLOG.INFO, 'Tcin timestamp is: %s' % str(tcin))
                self._tcin = tcin
            else:
                if tcin == 'None':
                    #zLOG.LOG('Media Clip', zLOG.INFO, 'Tcin is None')
                    self._tcin = None
                elif tcin == '':
                    #zLOG.LOG('Media Clip', zLOG.INFO, 'Tcin is empty string')
                    self._tcin = TimeStamp()
                else:
                    #zLOG.LOG('Media Clip', zLOG.INFO, 'Tcin contains something')
                    zLOG.LOG('Media Clip', zLOG.INFO, 'Before tcin set %s' % str(tcin))
                    tcin = TimeStamp(tcin)
                    tcin = tcin.millis()
                    tcin = tcin - offset
                    tcin = TimeStamp(tcin)
                    self._tcin = tcin
                    zLOG.LOG('Media Clip', zLOG.INFO, 'After tcin set %s' % str(self._tcin))

        if tcout is not None:
            #zLOG.LOG('Media Clip', zLOG.INFO, 'Got tcout. Tcout is %s' % str(tcout))
            if General.isinstance(tcout, TimeStamp):
                tcout = tcout.millis()
                tcout = tcout - offset
                tcout = TimeStamp(tcout)
                self._tcout = tcout
            else:
                if tcout == 'None':
                    self._tcout = None
                elif tcout == '':
                    self._tcout = TimeStamp()
                else:
                    tcout = TimeStamp(tcout)
                    tcout = tcout.millis()
                    tcout = tcout - offset
                    self._tcout = TimeStamp(tcout)

    def tcInString(self):
        """
        Returns tcin
        """
        return str(self._tcin)

    def tcOutString(self):
        """
        Returns tcout
        """
        return str(self._tcout)

    def SearchableText(self):
        """
            text for indexing
        """
        return "%s %s" % (self.title, self.description)

    security.declareProtected(CMFCorePermissions.ManageProperties, 'edit')
    def edit(self, title=None, description=None, effective_date=None, expiration_date=None, tcin=None, tcout=None, masterfile=None):
        """
        Edit the settings for the clip
        """
        if title is not None:
            self.setTitle(title)

        if description is not None:
            self.setDescription(description)

        if effective_date is not None:
            self.setEffectiveDate(effective_date)

        if expiration_date is not None:
            self.setExpirationDate(expiration_date)

        self.setTimeCodes(tcin, tcout)
            
        if masterfile is not None:
            self.setMasterFilePath(masterfile)

        self.reindexObject()


    security.declarePublic('syn_url')
    def syn_url(self):
        """
        Return the url to be syndicated
        """
        url = "%s?start=%s&end=%s" % (self.masterfile.lookupPlayerSource(), self.tcin, self.tcout)
        #zLOG.LOG('Clip', zLOG.INFO, 'syn_url must be changed to use media tool')
        return url
        #e_obj = self.restrictedTraverse('..')
        #if e_obj.usePixregion == 'true':
        #   src = 'SmilGenPix.smil'
        #else:
        #   src = 'SmilGen.smil'
        #return "%s/%s" % (self.absolute_url(), src)


def addMediaClip(self, id, REQUEST=None):
    ob=MediaClip(id)
    self._setObject(id, ob)

Globals.InitializeClass(MediaClip)
