##############################################################################
#    Copyright (C) 2003  MM Manager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################


ADD_CONTENT_PERMISSION = 'Add portal content'

import Globals
from Globals import package_home
import zLOG
import os
from Products.CMFCore import CMFCorePermissions
from AccessControl import ClassSecurityInfo
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.utils import getToolByName
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFDefault.utils import *

from Products.MyMediaManager.General import webcolors
from DateTime import DateTime
from Acquisition import aq_base, aq_parent, aq_inner
import Products.MyMediaManager.General
from Products.MyMediaManager.MediaSkinBase import MediaSkinExtensionBase

factory_type_information = (
              {'id': 'Media Player'
             , 'meta_type': 'Media Manager Player Skin'
             , 'description': 'A Media Player is used to design the embedded player'
             , 'product': 'MyMediaManager'
             , 'icon': 'mediaplayer.gif'
             , 'factory': 'addMediaPlayer'
             , 'filter_content_types': 0
             , 'immediate_view': 'mediaplayer_edit_form'
             , 'actions': ({'name': 'View'
                          , 'id': 'view'
                          , 'action': 'mediaplayer_view'
                          , 'permissions': (CMFCorePermissions.View,)
                          , 'category': 'object'}
                         , {'name': 'Edit'
                          , 'id': 'player_edit'
                          , 'action': 'mediaplayer_edit_form'
                          , 'permissions': (CMFCorePermissions.ModifyPortalContent,)
                          , 'category': 'object'}
                         , {'name': 'Metadata'
                          , 'id': 'metadata'
                          , 'action': 'metadata_edit_form'
                          , 'permissions': (CMFCorePermissions.ModifyPortalContent,)
                          , 'category': 'object'})
            }
              , {'id': 'Media Playlist'
               , 'meta_type': 'Media Manager Playlist Skin'
               , 'description': 'A Media Playlist is used to design the playlist'
               , 'product': 'MyMediaManager'
               , 'icon': 'mediaplaylist.gif'
               , 'factory': 'addMediaPlaylist'
               , 'filter_content_types': 0
               , 'immediate_view': 'mediaplaylist_edit_form'
               , 'actions': ({'name': 'View'
                            , 'id': 'view'
                            , 'action': 'mediaplaylist_view'
                            , 'permissions': (CMFCorePermissions.View,)
                            , 'category': 'object'}
                           , {'name': 'Edit'
                            , 'id': 'playlist_edit'
                            , 'action': 'mediaplaylist_edit_form'
                            , 'permissions': (CMFCorePermissions.ModifyPortalContent,)
                            , 'category': 'object'}
                           , {'name': 'Metadata'
                            , 'id': 'metadata'
                            , 'action': 'metadata_edit_form'
                            , 'permissions': (CMFCorePermissions.ModifyPortalContent,)
                            , 'category': 'object'})
                })

class MediaPlayer(MediaSkinExtensionBase):
    """
        A Media Player Skin
    """

    __implements__ = ( PortalContent.__implements__
                     , DefaultDublinCoreImpl.__implements__
                     )

    meta_type='Media Manager Player Skin'

    effective_date = expiration_date = None
    _isDiscussable = 0
    extra_width = 0
    extra_height = 0

    _default_content_fn = os.path.join(package_home(globals()),
                                       'www', 'mediaplayer.html')

    security = ClassSecurityInfo()

    def __init__(self, id, title='', description=''):
        """
        constructor
        """
        MediaSkinExtensionBase.__init__(self, id, title=title, description=description)
        zLOG.LOG('Media Player Skin init', zLOG.INFO, 'id: %s, title: %s' % (id, title))
        self.id = id
        self.title = title
        self.description = description
        self.extra_width = 0
        self.extra_height = 0

    security.declareProtected(CMFCorePermissions.ManageProperties, 'edit')
    def edit(self, title=None, text='', extra_width=None, extra_height=None):
        if title is not None:
           self.setTitle(title)

        if extra_width is not None:
            self.extra_width = int(extra_width)

        if extra_height is not None:
            self.extra_height = int(extra_height)

        text = bodyfinder(text)
        self._edit(text=text)

    def player_width(self, REQUEST=None, extra_width=0):
        """
        Find the width of the player
        """
        if not REQUEST:
            REQUEST = self.REQUEST

        servinfopath = REQUEST.get('servinfopath', None)
        if servinfopath:
            portal = getToolByName(self, 'portal_url').getPortalObject()
            serverinfo = portal.restrictedTraverse(servinfopath)
            event = serverinfo.aq_parent
            mnet = getToolByName(self, 'portal_medianet')
            width = 0
            if mnet.isLive(event):
                encprofile = serverinfo.getEncoderProfile()
                width = int(encprofile.getValue('width')) + extra_width
            else:
                width = int(serverinfo.getValueFor('width')) + extra_width
        else:
            width = 240 + extra_width

        return width

    def player_height(self, REQUEST=None, extra_height=0):
        """
        Find the height of the player
        """
        if not REQUEST:
            REQUEST = self.REQUEST

        servinfopath = REQUEST.get('servinfopath', None)
        if servinfopath:
            portal = getToolByName(self, 'portal_url').getPortalObject()
            serverinfo = portal.restrictedTraverse(servinfopath)
            event = serverinfo.aq_parent
            mnet = getToolByName(self, 'portal_medianet')
            height = 0
            if mnet.isLive(event):
                encprofile = serverinfo.getEncoderProfile()
                height = int(encprofile.getValue('height')) + extra_height
            else:
                height = int(serverinfo.getValueFor('height')) + extra_height
        else:
            height = 240 + extra_height

        return height

    def playerurl (self, REQUEST=None):
        """
        Find the url to put into filename attribute
        """
        if not REQUEST:
            REQUEST = self.REQUEST

        servinfopath = REQUEST.get('servinfopath', None)
        contentpath = REQUEST.get('contentpath', None)
        if servinfopath and contentpath:
            portal = getToolByName(self, 'portal_url').getPortalObject()
            serverinfo = portal.restrictedTraverse(servinfopath)
            content = portal.restrictedTraverse(contentpath)
            zLOG.LOG("%s - playerurl" % self.absolute_url(), zLOG.INFO, 'Content has meta_type %s' % content.meta_type)
            mediaserver = serverinfo.getMediaServer()
            metagen = mediaserver.defaultMetacodeGenerator
            playurl = "%s/%s?servinfoid=%s" % (content.absolute_url(), metagen, serverinfo.getId())
            zLOG.LOG("%s - playerurl" % self.absolute_url(), zLOG.INFO, 'Playerurl is %s' % playurl)
            return playurl
        else:
            return self.absolute_url()

    def content_url(self, REQUEST=None):
        """
        Find the url to the content viewed
        """
        contentpath = REQUEST.get('contentpath', None)
        contenturl = None
        if contentpath:
            portal = getToolByName(self, 'portal_url').getPortalObject()
            try:
                content = portal.restrictedTraverse(contentpath)
                contenturl = content.absolute_url()
            except:
                pass
        return contenturl


def addMediaPlayer(self, id, title='', REQUEST=None):
    """
    Add a slide show
    """
    ob=MediaPlayer(id,title)
    self._setObject(id, ob)
    zLOG.LOG('MediaPlayer ', zLOG.INFO, 'addMediaPlayer')

class MediaPlaylist(MediaSkinExtensionBase):
    """
        A Media Playlist Skin
    """

    __implements__ = ( PortalContent.__implements__
                     , DefaultDublinCoreImpl.__implements__
                     )

    meta_type='Media Manager Playlist Skin'

    effective_date = expiration_date = None
    _isDiscussable = 0
    width = 0
    height = 0
    fgcolor = "black"
    bgcolor = "white"
    colors = webcolors

    _default_content_fn = os.path.join(package_home(globals()),
                                       'www', 'mediaplaylist.html')

    security = ClassSecurityInfo()

    def __init__( self, id, title='', description=''):
        """
        constructor
        """
        MediaSkinExtensionBase.__init__(self, id, title=title, description=description)
        zLOG.LOG('Media Playlist Skin init', zLOG.INFO, 'id: %s, title: %s' % (id, title))
        self.id = id
        self.title = title
        self.description = description
        self.width = 0
        self.height = 0
        self.fgcolor = "black"
        self.bgcolor = "white"

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'edit')
    def edit(self, title=None, text='', width=None, height=None, fgcolor=None, bgcolor=None):
        if title is not None:
           self.setTitle(title)

        if width is not None:
            self.width = width

        if height is not None:
            self.height = height

        if fgcolor is not None:
            self.fgcolor = fgcolor

        if bgcolor is not None:
            self.bgcolor = bgcolor

        text = bodyfinder(text)
        self._edit(text=text)
        self.reindexObject()

    def event_data(self, REQUEST=None):
        """
        Return a doctionary containing data about the event
        """
        if not REQUEST:
            REQUEST = self.REQUEST

        servinfopath = REQUEST.get('servinfopath', None)
        skinpath = REQUEST.get('cinemaskinpath', None)
        data = {}
        if servinfopath and skinpath:
            portal = getToolByName(self, 'portal_url').getPortalObject()
            serverinfo = portal.restrictedTraverse(servinfopath)
            mediaskin = portal.restrictedTraverse(skinpath)
            event = serverinfo.aq_parent
            medianet = getToolByName(self, 'portal_medianet')
            isLive = medianet.isLive(event)
            data['icon'] = event.getIcon()
            data['title'] = event.title_or_id()
            data['description'] = event.Description()
            if isLive:
                data['start'] = event.start().strftime('%d/%m-%Y %H:%M')
                data['end'] = event.end().strftime('%d/%m-%Y %H:%M')
            player = mediaskin.getPlayerSkin()
            if isLive or len(event.listClips()) == 0:
                data['playurl'] = "%s/mediaplayer_iframe_view?servinfopath=%s&contentpath=%s&cinemaskinpath=%s&referer_partner=%s" % \
                    (player.absolute_url(relative=1)
                   , servinfopath
                   , event.absolute_url(relative=1)
                   , mediaskin.absolute_url(relative=1)
                   , REQUEST.get('referer_partner', ''))
        else:
            data['icon'] = 'mediaplaylist.gif'
            data['title'] = self.title_or_id()
            data['description'] = self.Description()
            data['start'] = DateTime().strftime('%d/%m-%Y %H:%M')
            data['end'] = DateTime().strftime('%d/%m-%Y %H:%M')
            data['playurl'] = "%s/view" % self.absolute_url()

        return data

    def clip_datas(self, REQUEST=None):
        """
        Return a list of dictionaries containing clip data
        """
        if not REQUEST:
            REQUEST = self.REQUEST

        servinfopath = REQUEST.get('servinfopath', None)
        skinpath = REQUEST.get('cinemaskinpath', None)
        data = []
        if servinfopath and skinpath:
            portal = getToolByName(self, 'portal_url').getPortalObject()
            serverinfo = portal.restrictedTraverse(servinfopath)
            mediaskin = portal.restrictedTraverse(skinpath)
            event = serverinfo.aq_parent
            medianet = getToolByName(self, 'portal_medianet')
            isLive = medianet.isLive(event)
            player = mediaskin.getPlayerSkin()
            if not isLive:
                for clip in event.listClips():
                    clipdata = {}
                    clipdata['icon'] = clip.getIcon()
                    clipdata['title'] = clip.title_or_id()
                    clipdata['description'] = clip.Description()
                    clipdata['duration'] = str(clip.getDuration())[:-4]
                    clipdata['playurl'] = "%s/mediaplayer_iframe_view?servinfopath=%s&contentpath=%s&cinemaskinpath=%s&referer_partner=%s" % \
                        (player.absolute_url(relative=1)
                       , servinfopath
                       , clip.absolute_url(relative=1)
                       , mediaskin.absolute_url(relative=1)
                       , REQUEST.get('referer_partner', ''))
                    data.append(clipdata)
        else:
            clipdata ={}
            clipdata['icon'] = 'mediaplaylist.gif'
            clipdata['title'] = self.title_or_id()
            clipdata['description'] = self.Description()
            clipdata['duration'] = '00:00'
            clipdata['playurl'] = "%s/view" % self.absolute_url()
            data.append(clipdata)

        return data

def addMediaPlaylist(self, id, title='', REQUEST=None):
    """
    Add a slide show
    """
    ob=MediaPlaylist(id,title)
    self._setObject(id, ob)
    zLOG.LOG('MediaPlaylist ', zLOG.INFO, 'addMediaPlaylist')

Globals.InitializeClass(MediaPlayer)
Globals.InitializeClass(MediaPlaylist)
