##############################################################################
#    Copyright (C) 2003  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

#Python
import os, urllib

#Globals
from DateTime import DateTime
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo

#CMF
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.WorkflowCore import WorkflowAction #, afterCreate
from Products.CMFCore import CMFCorePermissions

factory_type_information = ({'id': 'Sub Event'
                                         , 'icon': 'event_icon.gif'
                                         , 'meta_type': 'Media Manager Sub Event'
                                         , 'description': 'Sub events can be used to schedule your live event'
                                         , 'product': 'MyMediaManager'
                                         , 'factory': 'addSubEvent'
                                         , 'immediate_view': 'subevent_edit_form'
                                         , 'actions': ({'id': 'view'
                                                         , 'name': 'View'
                                                         , 'action': 'subevent_view'
                                                         , 'permissions': (CMFCorePermissions.View,)
                                                         , 'category': 'object'},)
                                          },)

def addSubEvent(self, id, title=''):
    """
    Create an empty sub event.
    """
    subevent = SubEvent(id, title)
    self._setObject(id, subevent)

def _dateStrings( when ):

    strings = {}
    if when is not None:
        strings[ 'year' ]   = str( when.year() )
        strings[ 'month' ]  = str( when.month() )
        strings[ 'day' ]    = str( when.day() )
    else:
        strings[ 'year' ]   = ''
        strings[ 'month' ]  = ''
        strings[ 'day' ]    = ''
    return strings

class SubEvent(PortalContent, DefaultDublinCoreImpl):
    """
    Events are objects for the Calendar topical query.
    """
    meta_type='Media Manager Sub Event'
    portal_type='Object'
    converted_clip_path = ''

    # Declarative security
    security = ClassSecurityInfo()
    security.declareObjectProtected(CMFCorePermissions.View)

    def __init__(self, id, title='', description='', start_date = DateTime(), end_date = DateTime()):
        DefaultDublinCoreImpl.__init__(self)
        self.id=id
        self.setTitle(title)
        self.setDescription(description)
        self.effective_date = None
        self.expiration_date = None
        self.setStartDate(start_date)
        self.setEndDate(end_date)

    security.declarePrivate( '_datify' )
    def _datify( self, attrib ):
        if attrib == 'None':
            attrib = None
        elif not isinstance( attrib, DateTime ):
            if attrib is not None:
                attrib = DateTime( attrib )
        return attrib

    security.declarePublic('setConvertedClipPath')
    def setConvertedClipPath(self, clip_path):
        if clip_path is not None:
            self.converted_clip_path = clip_path

    security.declarePublic('getEndStrings')
    def getEndStrings(self):
        """
        """
        return _dateStrings(self.end())

    security.declarePublic('getStartStrings')
    def getStartStrings(self):
        """
        """
        return _dateStrings(self.start())

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'edit')
    def edit(self, title=None, description=None, effectiveDate=None, expirationDate=None, start_time=None, stop_time=None):
        """
        Edit the sub event
        """
        if title is not None:
            self.setTitle(title)
        if description is not None:
            self.setDescription(description)

        if start_time:
            efdate = '%s %s' % (effectiveDate, start_time)
            self.setStartDate(DateTime(efdate))

        if stop_time:
            exdate = '%s %s' % (expirationDate, stop_time)
            self.setEndDate(DateTime(exdate))

        self.reindexObject()
    edit = WorkflowAction(edit)

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'setStartDate')
    def setStartDate(self, start):
        """
        Setting the sub event start date, when the event is scheduled to begin.
        """
        self.start_date = self._datify(start)

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'setEndDate')
    def setEndDate(self, end):
        """
        Setting the sub event end date, when the event ends.
        """
        self.end_date = self._datify(end)

    security.declarePublic('start')
    def start(self):
        """
            Return our start time as a string.
        """
        date = getattr( self, 'start_date', None )
        return date is None and self.created() or date

    security.declarePublic('end')
    def end(self):
        """
            Return our stop time as a string.
        """
        date = getattr( self, 'end_date', None )
        return date is None and self.start() or date

    security.declarePublic('getStartTimeString')
    def getStartTimeString( self ):
        """
            Return our start time as a string.
        """
        return self.start().strftime('%H:%M:%S')

    security.declarePublic('getStopTimeString')
    def getStopTimeString( self ):
        """
            Return our stop time as a string.
        """
        return self.end().strftime('%H:%M:%S')

InitializeClass(SubEvent)
